/*
 * Copyright (C) 2022 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <string.h>
#include <sys/param.h>

#define DATA_SIZE 8

/**
 * Boolean, the index is incremented if set.
 * This is a variable to be shared with TCLspice.
 */
unsigned int isEnabled = 0;

char theData[DATA_SIZE];    //!< data to be transmitted bit after bit
unsigned int theIndex = 0;  //!< index of bit to obtain with \ref getBit().

/**
 * Changes the value of the data to be transmitted ; resets the index.
 * This is a function to be shared with TCLspice.
 */
void setData(
  char *theAdress,      //!< address of the data to overwrite from
  unsigned int theSize  //!< number of Bytes to copy to data
) {
  memset(&theData[0], 0xff, DATA_SIZE);
  memcpy(&theData[0], theAdress, MIN(theSize, DATA_SIZE));
  theIndex = 0;
}

/**
 * Returns the value of the next bit. Increments the bit index if boolean
 * isEnabled is set.
 * This is a function to be shared with TCLspice.
 */
unsigned int getBit(
  void
) {
  char theValue;
  theIndex = (theIndex % (8*DATA_SIZE));          // to avoid overflows
  theValue = theData[theIndex / 8];               // select the Byte
  theValue = (theValue >> (theIndex % 8)) & 0x01; // select the bit in Byte
  if ( isEnabled != 0 ) {
    theIndex++;
  }
  return theValue;
}

