#!/bin/sh
# the next line restarts using tclsh \
exec tclsh "$0" ${1+"$@"}

#
# Copyright (C) 2022 Jonathan Certes
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

# load our module for the software:
set dir [file dirname [file normalize [info script]]]
load [file join ${dir} "software.so"] foreign

# load ngspice module ; this provides TCLSpice:
source "/usr/share/ngspice/pkgIndex.tcl"
load /usr/lib/x86_64-linux-gnu/libspice.so

# packages to plot data:
package require Tk
package require BLT

puts "When we start, variable \"isEnabled\" is set to: ${isEnabled}"

# set the data in C:
set theData "HELLO"
setData ${theData} [string length ${theData}]

# to store the whole data to plot:
blt::vector create theTime
blt::vector create thePlot
# to store the data of one step:
blt::vector create theTmp

# load the hardware with TCLspice:
spice::source "hardware.cir"

# set the value of the C shared variable:
set isEnabled 1

# for each bit transmitted, we alter the DC of the input source and perform an
# operating point analysis with spice.
for {set i 0} {$i < 8*[string length ${theData}]} {incr i} {
  spice::alter  "vin" dc = [getBit]
  spice::op
  # add the measure of V(OUT) to the vector to plot:
  theTime append [expr {(2 * $i) * 1e-6}]
  spice::vectoblt "v(out)" theTmp
  thePlot append theTmp
  theTime append [expr {(2 * $i + 1) * 1e-6}]
  thePlot append theTmp
  # deactivate C data after 3 Bytes:
  if { $i >= 8*3 } {
    set isEnabled 0
  }
}

# plot the data, we expect V(OUT) to be set between 0 and 0.5 due to the
# voltage divider:
blt::graph .graph -title "V(OUT)" -height 2i
.graph element create "V(OUT)" \
  -color green -xdata theTime -ydata thePlot -pixels 2
pack .graph -fill x

